/*
 *  Gtk-Parted - a front-end for libparted
 *  Copyright (C) 2000, 2001  Damien Genet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * gtk refresh thread functions
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <pthread.h>
#include <string.h>
#include <errno.h> 

#include <gtk/gtk.h>

#include "i18n.h"
#include "gtk_thread.h"

static pthread_t running_thread = 0;
static pthread_mutex_t gtk_ref_count_mutex;
static pthread_cond_t gtk_ref_count_cond;
static gshort gtk_ref_count = 0;

void
gtk_thread_inc_ref_count               (void)
{
	g_return_if_fail (running_thread);
	
	pthread_mutex_lock (&gtk_ref_count_mutex);
	gtk_ref_count++;
	if (gtk_ref_count)
		pthread_cond_broadcast (&gtk_ref_count_cond);
	printf ("gtk_main_thread inc ref : %i\n", gtk_ref_count);
	pthread_mutex_unlock (&gtk_ref_count_mutex);
}

void
gtk_thread_dec_ref_count               (void)
{
	g_return_if_fail (running_thread);
	//g_return_if_fail (gtk_ref_count);
	
	pthread_mutex_lock (&gtk_ref_count_mutex);
	gtk_ref_count--;
	if (!gtk_ref_count)
		pthread_cond_broadcast (&gtk_ref_count_cond);
	printf ("gtk_main_thread dec ref : %i\n", gtk_ref_count);
	pthread_mutex_unlock (&gtk_ref_count_mutex);
}

static void
_gtk_main_thread                       (void)
{
	g_return_if_fail (running_thread);
	
	pthread_setcancelstate (PTHREAD_CANCEL_ENABLE,
	                        NULL);

	while ( TRUE ) {
		pthread_mutex_lock (&gtk_ref_count_mutex);
		if (gtk_ref_count) {
			printf ("gtk_main_thread iterate\n");
			gtk_main_iteration ();
		} else {
			printf ("gtk_main_thread waiting for gtk_ref_count_cond\n");
			pthread_cond_wait (&gtk_ref_count_cond,
			                   &gtk_ref_count_mutex);
		}
		pthread_mutex_unlock (&gtk_ref_count_mutex);
		pthread_testcancel ();
	}
}

gboolean
gtk_thread_start                       (void)
{
	g_return_val_if_fail (!running_thread, FALSE);
	
	if ( pthread_mutex_init (&gtk_ref_count_mutex, NULL) ) {
		g_printerr (_("%s: initialising mutex\n"),
		            g_strerror (errno));
		return FALSE;
	}
	
	if ( pthread_cond_init (&gtk_ref_count_cond, NULL) ) {
		g_printerr (_("%s: initialising condition\n"),
		            g_strerror (errno));
		return FALSE;
	}
	
	if ( pthread_create (&running_thread, NULL, (void*) _gtk_main_thread, NULL) < 0 ) {
		g_printerr ("%s: creating thread\n",
		            g_strerror (errno));
		return FALSE;
	}
	
	printf ("gtk_main_thread started\n");
	
	return TRUE;
}

gboolean
gtk_thread_stop                        (void)
{
	g_return_val_if_fail (running_thread, FALSE);
	
	if ( pthread_cancel (running_thread) != 0 ) {
	 	g_printerr ("%s: canceling thread\n",
		            g_strerror (errno));
		return FALSE;
	}		
	
	pthread_cond_broadcast (&gtk_ref_count_cond);
			
	if ( pthread_join (running_thread, NULL) != 0 ) {
	 	g_printerr ("%s: joining thread\n",
		            g_strerror (errno));
		return FALSE;
	}
	
	printf ("gtk_main_thread stopped\n");
	
	running_thread = 0;
	return TRUE;
}
